#define _XTAL_FREQ 16000000
#include <xc.h>

#pragma config CLKOUTEN = OFF
#pragma config WDTE = OFF // or control in software - SWDTEN
#pragma config PWRTE = OFF
#pragma config CP = OFF
#pragma config BOREN = OFF
#pragma config MCLRE = OFF
#pragma config FOSC = INTOSC

#pragma config STVREN = ON
#pragma config LPBOR = OFF
#pragma config BORV = LO
#pragma config LVP = OFF
#pragma config WRT = ALL // we don't need self write

typedef struct{
		unsigned short PWMDC;
		unsigned char PWMCON;
} pwmReg_t;
pwmReg_t pwmReg[3] @0x611;

unsigned char pwm1;
// we don't use pwm2
unsigned char pwm3;
unsigned char pwm4;
signed char addend;


void init(){

	// IO setup

	// RA0 - IN
	// RA1 - IN
	// RA2 - OUT - LED
	// RA3 - IN - button
	// RA4 - OUT - LED
	// RA5 - OUT - LED
	TRISA = 0x0B; // 0000 1011
	ANSELA = 0; // all pins are digital
    OPTION_REGbits.nWPUEN = 0; // enable weak pull-ups
	WPUA = 0x08; // pull up on RA3

	// PWM & Timer 2 setup
#define PWM_COM_SETUP (_PWM1CON_PWM1EN_MASK | _PWM1CON_PWM1OE_MASK \
              | _PWM1CON_PWM1OUT_MASK) & ~_PWM1CON_PWM1POL_MASK
	PR2 = 0x3F; // period
    T2CONbits.T2OUTPS = 0;
    T2CONbits.T2CKPS = 0;
	PWM1CON = PWM_COM_SETUP;
	PWM3CON = PWM_COM_SETUP;
	PWM4CON = PWM_COM_SETUP;
    T2CONbits.TMR2ON = 1;
}


void setPwm(unsigned char pwmCon, unsigned char pwmValue){
	union{
		struct{
			unsigned char B0;
			unsigned char B1;
		};
		unsigned short W;
	} word;
	word.B0 = 0;
	word.B1 = pwmValue;
	word.W >>= 2;

	pwmCon--;

	pwmReg[pwmCon].PWMDC = word.W;

}

void main(void){
	OSCCONbits.IRCF = 0xF; // 1111 = 16MHz
	OSCCONbits.SCS = 3; // internal clock
	init();

	while(1){
		if(PORTAbits.RA3){
			//press and hold the button to slowly turn the LEDs on
			if(pwm1>0){
				pwm1--;
				pwm3=pwm1;
				pwm4=pwm1;
			}
		}else{
			// release the button to slowly turn the LEDs off
			if(pwm1<255){
				pwm1++;
				pwm3=pwm1;
				pwm4=pwm1;
			}
		}
		
		__delay_ms(10);
		setPwm(1, pwm1);
		setPwm(3, pwm3);
		setPwm(4, pwm4);
	}
}
